#!/bin/bash
# (c) 2014-2015 Helge Jung <hej@c3pb.de>
#
# This script is controlled by environment variables, the
# first two being mandatory:
#
# BASE      = Gluon Version (tag or commit, i.e. v2014.4)
# BRANCH    = Firmware Branch (stable/testing/experimental)
# SITE      = specific site repository commit-id (leave blank to use HEAD)
# VERSION   = the version tag (can only be empty if BRANCH=experimental)
# BUILD_TS  = build timestamp (format: %Y-%m-%d %H:%M:%S)
# BROKEN    = 0 (default) or 1, build the untested hardware model firmwares, too
# MAKEJOBS  = number of compiler processes running in parallel (default: number of CPUs/Cores)
#

if [ "_$BRANCH" == "_" ]; then
	echo "Please specify BRANCH environment variable." >&2
	exit 1
fi

if [ "_$BASE" == "_" ]; then
	echo "Please specify BASE environment variable (Gluon, i.e. 'v2014.3' or commit-id)." >&2
	exit 1
fi

if [ "_$VERSION" == "_" -a "$BRANCH" != "experimental" ]; then
	echo "Please specify VERSION environment variable (not necessary for experimental branch)." >&2
	exit 1
fi

MY_DIR=$(dirname $0)
MY_DIR=$(readlink -f "$MY_DIR")
CODE_DIR="src"
pushd $MY_DIR > /dev/null
[ "_$BUILD_TS" == "_" ] && export BUILD_TS=$(date +"%Y-%m-%d %H:%M:%S")

. functions.sh

### CHECK THAT VERSION DOES NOT YET EXISTS
[ -n "$VERSION" -a -x "versions/${VERSION}" ] && abort "There exists a version file for '$VERSION' ... you are trying to do something really stupid, aren't you?"

### INIT /src IF NECESSARY
if [ ! -d "$CODE_DIR" ]; then
	info "Code directory does not exist yet - fetching Gluon ..."
	git clone https://github.com/freifunk-gluon/gluon.git "$CODE_DIR"
fi

### INIT /src/site IF NECESSARY
if [ ! -d "$CODE_DIR/site" ]; then
        info "Site repository does not exist, fetching it ..."
	git clone https://git.c3pb.de/freifunk-pb/site-ffpb.git "$CODE_DIR/site"
fi

### CHECKOUT GLUON
progress "Checking out GLUON '$BASE' ..."
cd $CODE_DIR
# TODO: check if gluon got modified and bail out if necessary
git fetch ; git checkout -q $BASE
[ "$?" -eq "0" ] || abort "Failed to checkout '$BASE' gluon base version, mimimi." >&2
GLUON_COMMIT=$(git rev-list --max-count=1 HEAD)

### CHECKOUT SITE REPO
progress "Checking out SITE REPO ..."
cd site
# TODO: check if site got modified locally and bail out if necessary
if [ "_$SITE" == "_" ]; then
	# no specific site given - get the most current one
	git checkout -q $BRANCH ; git pull
	[ "$?" -eq "0" ] || abort "Failed to get newest '$BRANCH' in site repository, mimimi."
else
	# commit given - use this one
	git checkout -q $SITE || abort "Failed to checkout requested site commit, mimimi."
fi
SITE_COMMIT=$(git rev-list --max-count=1 HEAD)

cd ..

### CLEAN
if [ "$BRANCH" != "experimental" ]; then
	progress "Cleaning your build environment ..."
	make clean
fi

### PREPARE
[ -n "${MAKEJOBS}" ] || MAKEJOBS=$(grep -c "^processor" /proc/cpuinfo)
progress "Preparing the build environment (make update) ..."
make update
[ "$?" -eq "0" ] || abort "Failed to update the build environment, mimimi."

### BUILD TOOLCHAIN
progress "Building toolchain if necessary (this is not possible on a fresh build) ..."
faketime "$BUILD_TS" make toolchain -j ${MAKEJOBS}

### BUILD FIRMWARE
progress "Building the firmware - please stand by!"
[ "_$BROKEN" == "_" ] && export BROKEN=0
if [ "$BRANCH" != "experimental" ]; then
	GLUON_BRANCH=$BRANCH GLUON_RELEASE=$VERSION BROKEN=$BROKEN faketime "$BUILD_TS" make -j ${MAKEJOBS}
else
	GLUON_BRANCH=experimental faketime "$BUILD_TS" make -j ${MAKEJOBS}
fi

[ "$?" -eq "0" ] || abort "Failed to build the firmware, mimimi."

cd ..

# write build info
progress "Writing build info ..."
mkdir -p "output/${BRANCH}"
echo -en "" > "output/${BRANCH}/build_info.txt"
echo "VERSION=${VERSION}" >> "output/${BRANCH}/build_info.txt"
echo "GLUON=${GLUON_COMMIT} # ${BASE}" >> "output/${BRANCH}/build_info.txt"
echo "BRANCH=${BRANCH}" >> "output/${BRANCH}/build_info.txt"
echo "SITE=${SITE_COMMIT} # ${VERSION}" >> "output/${BRANCH}/build_info.txt"
echo "TS=${BUILD_TS}" >> "output/${BRANCH}/build_info.txt"

# compress all binaries into 7z archive
progress "Assembling images.7z ..."
[ -e "output/${BRANCH}/images.7z" ] && rm "output/${BRANCH}/images.7z"
7z a "output/${BRANCH}/images.7z" ${CODE_DIR}/images/sysupgrade/*.bin ${CODE_DIR}/images/factory/*.bin || abort "Failed to assemble images (did you install p7zip-full?)."

# generate and copy manifest
progress "Generating and copying manifest ..."
pushd $CODE_DIR
GLUON_BRANCH=$BRANCH make manifest || abort "Failed to generate the manifest, try running 'make manifest' in '$CODE_DIR' directory manually."
popd
cp "${CODE_DIR}/images/sysupgrade/${BRANCH}.manifest" "output/${BRANCH}/"

# write (copy) version file
cp "output/${BRANCH}/build_info.txt" "versions/${VERSION}"

# The end. Finally.
success "We're done, go and checkout your new firmware in output/${BRANCH}!"
popd > /dev/null

